@@ -1,10 +1,9 @@
-from __future__ import print_function, unicode_literals, absolute_import, division
-
+import os
 from reahl.sqlalchemysupport import SqlAlchemyControl
 
+reahlsystem.connection_uri = os.environ.get('REAHL_TEST_CONNECTION_URI', 'postgresql:///reahl')
 reahlsystem.root_egg = 'reahl-domain'
 #reahlsystem.connection_uri = 'mysql:///reahl'
-reahlsystem.connection_uri = 'postgresql:///reahl'
 #reahlsystem.connection_uri = 'sqlite:////tmp/test.db'
 
 reahlsystem.orm_control = SqlAlchemyControl(echo=False)
