@@ -1,4 +1,4 @@
-# Copyright 2013-2018 Reahl Software Services (Pty) Ltd. All rights reserved.
+# Copyright 2013-2020 Reahl Software Services (Pty) Ltd. All rights reserved.
 #
 #    This file is part of Reahl.
 #
@@ -14,15 +14,19 @@
 #    You should have received a copy of the GNU Affero General Public License
 #    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
-"""A collection of classes to deal with accounts for different parties on a system."""
+"""A collection of classes to deal with accounts for different parties on a system.
+
+
+Run 'reahl componentinfo reahl-domain' for information on configuration.
+
+"""
 
-from __future__ import print_function, unicode_literals, absolute_import, division
-import six
 from datetime import datetime, timedelta
 import passlib.context
 import re
 import random
 from string import Template
+import logging
 
 from sqlalchemy import Column, Integer, ForeignKey, UnicodeText, String, DateTime, Boolean, Unicode
 from sqlalchemy.orm import relationship, reconstructor
@@ -43,26 +47,36 @@ from reahl.domain.workflowmodel import DeferredAction, Requirement
 _ = Catalogue('reahl-domain')
 
 class SystemAccountConfig(Configuration):
-    filename = 'systemaccountmodel.config.py'
-    config_key = 'accounts'
+    """Configuration containing general system information
+    
+    All templates in this configuration are PEP292 formatted strings that can use the following substitutions:
 
-    """Configuration containing general system information"""
-    admin_email = ConfigSetting(default='admin@example.com', description='The email address from which automated emails are sent', dangerous=True)
-    request_verification_timeout = ConfigSetting(default=10, description='The time an unvalidated request is kept alive')
-    activation_subject = ConfigSetting(default='''Your registration for $email''', description='Subject used for activation emails')
+    $email  
+        The email address to which the message applies
+
+    $secret 
+        The relevant secret key
+    """
+
+    filename = 'systemaccountmodel.config.py' 
+    config_key = 'accounts'                   
+
+    admin_email = ConfigSetting(default='admin@example.com', description='The email address from which automated emails are sent', dangerous=True) 
+    request_verification_timeout = ConfigSetting(default=10, description='The time an unvalidated request is kept alive') 
+    activation_subject = ConfigSetting(default='''Your registration for $email''', description='A Subject used for activation emails') 
     activation_email = ConfigSetting(default='''You, or someone on your behalf requested to be registered using $email.\n\n'''\
         '''If you have received this message in error, please ignore it.\n\n'''\
-        '''Your secret key is: $secret_key\n\n''', description='Body of an activation email')
-    new_password_subject = ConfigSetting(default='''Your password for $email''', description='Subject used for password reset emails')
+        '''Your secret key is: $secret_key\n\n''', description='Body of an activation email') 
+    new_password_subject = ConfigSetting(default='''Your password for $email''', description='Subject used for password reset emails') 
     new_password_email = ConfigSetting(default='''You, or someone on your behalf requested to pick a new password for $email.\n\n'''\
         '''If you have received this message in error, please ignore it.\n\n'''\
-        '''Your secret key is: $secret_key\n\n''', description='Body of a password reset email')
-    email_change_subject = ConfigSetting(default='''Your new email $email''', description='Subject used for email address change emails')
+        '''Your secret key is: $secret_key\n\n''', description='Body of a password reset email') 
+    email_change_subject = ConfigSetting(default='''Your new email $email''', description='Subject used for email address change emails') 
     email_change_email = ConfigSetting(default='''You, or someone on your behalf requested to change your email to $email.\n\n'''\
         '''If you have received this message in error, please ignore it.\n\n'''\
         '''You need to accept this change before it will take effect.\n\n'''\
-        '''Your secret key is: $secret_key\n\n''', description='Body of an email address changed email')
-    mailer_class = ConfigSetting(default=Mailer, description='The class to instantiate for sending email')
+        '''Your secret key is: $secret_key\n\n''', description='Body of an email address changed email') 
+    mailer_class = ConfigSetting(default=Mailer, description='The class to instantiate for sending email') 
 
 
 class SystemAccount(Base):
@@ -125,7 +139,7 @@ class EmailAndPasswordSystemAccount(SystemAccount):
     email = Column(Unicode(254), nullable=False, unique=True, index=True) #: The email address of this account
 
     def __init__(self, **kwargs):
-        super(EmailAndPasswordSystemAccount, self).__init__(**kwargs)
+        super().__init__(**kwargs)
         self.init_on_load()
 
     @reconstructor
@@ -217,8 +231,6 @@ class EmailAndPasswordSystemAccount(SystemAccount):
                 self.disable()
             raise InvalidPasswordException(commit=True)
         if new_hash:
-            if six.PY2:
-                new_hash = new_hash.decode('utf-8')
             self.password_hash = new_hash
 
     def send_activation_notification(self):
@@ -242,8 +254,6 @@ class EmailAndPasswordSystemAccount(SystemAccount):
             raise InvalidEmailException()
 
         new_hash = self.crypt_context.hash(password)
-        if six.PY2:
-            new_hash = new_hash.decode('utf-8')
         self.password_hash = new_hash
 
     def request_email_change(self, new_email):
@@ -365,7 +375,7 @@ class AccountManagementInterface(Base):
 class RepeatPasswordField(PasswordField):
     def __init__(self, other_field, default=None, required=False, required_message=None, label=None):
         label = label or _('')
-        super(RepeatPasswordField, self).__init__(default, required, required_message, label)
+        super().__init__(default, required, required_message, label)
         self.add_validation_constraint(EqualToConstraint(other_field))
         
         
@@ -458,7 +468,7 @@ class VerificationRequest(Requirement):
 
     def __init__(self, **kwargs):
         self.generate_salt()
-        super(VerificationRequest, self).__init__(**kwargs)
+        super().__init__(**kwargs)
 
     def generate_salt(self):
         alphabet = 'ABCDEFGHIJKLMNOPQRSTUVWXYZqwertyuiopasdfghjklzxcvbnm0123456789'
@@ -472,7 +482,13 @@ class VerificationRequest(Requirement):
         message_text = Template(config.get_from_string(mail_config_key)).safe_substitute(data)
         message = MailMessage(admin_email, [destination], subject, message_text) 
         mailer = config.accounts.mailer_class.from_context()
-        mailer.send_message(message)
+        try:
+            mailer.send_message(message)
+        except ConnectionError as e:
+            message = _('Could not send registration verification email message to %s: %s.') % (self.email, str(e))
+            user_message = message + _(' Please contact the site administrator and try again later.')
+            logging.getLogger(__name__).error(message)
+            raise DomainException(message=user_message)
 
     def send_notification(self):
         pass
@@ -543,7 +559,7 @@ class ActivateAccount(DeferredAction):
         config = ExecutionContext.get_context().config
         deadline = datetime.now() + timedelta(days=config.accounts.request_verification_timeout)
         self.system_account = system_account
-        super(ActivateAccount, self).__init__(deadline=deadline, **kwargs)
+        super().__init__(deadline=deadline, **kwargs)
 
     def success_action(self):
         self.system_account.activate()
@@ -568,7 +584,7 @@ class ChangeAccountEmail(DeferredAction):
         config = ExecutionContext.get_context().config
         deadline = datetime.now() + timedelta(days=config.accounts.request_verification_timeout)
         self.system_account = system_account
-        super(ChangeAccountEmail, self).__init__(requirements=requirements,
+        super().__init__(requirements=requirements,
                                                  deadline=deadline)
 
     @property
@@ -595,7 +611,7 @@ class LoginSession(Base):
     __tablename__ = 'loginsession'
 
     id = Column(Integer, primary_key=True)
-    discriminator = Column('row_type', String(40))
+    discriminator = Column('row_type', String(length=40))
     __mapper_args__ = {'polymorphic_on': discriminator}
 
     account_id = Column(Integer, ForeignKey('systemaccount.id'), index=True)
@@ -622,10 +638,9 @@ class LoginSession(Base):
 
     def log_out(self):
         self.account = None
-                    
 
 
-class SystemAccountStatus(object):
+class SystemAccountStatus:
     def as_user_message(self):
         return _('Unknown status')
     def is_active(self):
